import datetime
import os
import sys
import time

sys.path.append('..')

import config
import sqlalchemy as sa
import pandas as pd
from sqlalchemy.orm import sessionmaker
from utillities.internal_code import InternalCode
from utillities.obs_util import ObsUtil


class BaseSync(object):
    def __init__(self, source_table=None, dest_table=None):
        source_db = '''mysql+mysqlconnector://{0}:{1}@{2}:{3}/{4}'''.format(config.hb_db_user, config.hb_db_pwd,
                                                                            config.hb_db_host, config.hb_db_port,
                                                                            config.hb_db_database)
        destination_db = '''mysql+mysqlconnector://{0}:{1}@{2}:{3}/{4}'''.format(config.rl_db_user,
                                                                                 config.rl_db_pwd,
                                                                                 config.rl_db_host,
                                                                                 config.rl_db_port,
                                                                                 config.rl_db_database)
        # 源数据库
        self.source = sa.create_engine(source_db)
        # 目标数据库
        self.destination = sa.create_engine(destination_db)
        # 目标数据库Session
        self.dest_session = sessionmaker(bind=self.destination, autocommit=False, autoflush=True)
        self.source_table = source_table
        self.dest_table = dest_table
        self.internal_code = InternalCode()
        # self.utils = TmImportUtils(self.source, self.destination, self.source_table, self.dest_table)
        self.obs_util = ObsUtil()

    def get_start_date(self, table=None, type='trade_date'):
        if table is None:
            table = self.dest_table
        sql = """select max({0}) as trade_date from `{1}` where flag=1;""".format(type, table)
        trades_sets = pd.read_sql(sql, self.destination)
        td = 20070101
        if not trades_sets.empty:
            ts = trades_sets['trade_date'][0]
            if ts is None or ts == 'None':
                return td
            td = str(ts).replace('-', '')
        return td

    def check_data_exist(self, table, filter):
        sql = """select count(1) as cou from `{0}` where {1} and flag=1;""".format(table, filter)
        trades_sets = pd.read_sql(sql, self.destination)
        if not trades_sets.empty:
            ts = trades_sets['cou'][0]
            if ts > 0:
                return True
        return False

    def get_start_date_by_uplog(self):
        sql = """select * from `{0}` where task_name='{1}' and flag=1;""".format('data_update_log', self.dest_table)
        trades_sets = pd.read_sql(sql, self.destination)
        td = 20070101
        if not trades_sets.empty:
            ts = trades_sets['max_tag'][0]
            if ts is None or ts == 'None':
                return td
            td = str(ts)
        return td

    def delete_trade_data(self, trade_date, date_type='trade_date'):
        session = self.dest_session()
        # session.execute('''delete from `{0}` where trade_date={1}'''.format(self.dest_table, trade_date))
        session.execute(
            '''update `{0}` set flag = null,is_verify=2 where {1}={2} and flag=1'''.format(self.dest_table, date_type,
                                                                                           trade_date))
        session.commit()

    def delete_begin_end(self, start_date, end_date):
        session = self.dest_session()
        sql = f"update `{self.dest_table}` set flag = null,is_verify=2 where trade_date between {start_date} and {end_date} and flag=1"
        session.execute(sql)
        session.commit()

    def create_table(self, create_sql, table=None, model=None, unique_key=None):
        if table is None:
            table = self.dest_table
        drop_sql = """drop table if exists `{0}`;""".format(table)
        session = self.dest_session()
        session.execute(drop_sql)
        session.execute(create_sql)
        session.execute(
            '''alter table `{0}` add  `flag` int DEFAULT 1;'''.format(table))
        session.execute(
            '''alter table `{0}` add  `is_verify` int DEFAULT 0;'''.format(table))
        session.execute(
            '''alter table `{0}` add  `create_time` timestamp NULL DEFAULT CURRENT_TIMESTAMP;'''.format(table))
        session.execute(
            '''alter table `{0}` add  `update_time` timestamp NULL DEFAULT CURRENT_TIMESTAMP ON UPDATE CURRENT_TIMESTAMP;'''.format(
                table))
        session.execute('''CREATE INDEX {0}_flag_index ON `{0}` (flag);'''.format(table))
        session.execute('''CREATE INDEX {0}_is_verify_index ON `{0}` (is_verify);'''.format(table))
        session.execute('''CREATE INDEX {0}_update_time_index ON `{0}` (update_time);'''.format(table))
        if model == 'basic_info':
            session.execute('''alter table `{0}` add  `tmstamp` BIGINT	NOT NULL;'''.format(table))
            session.execute('''alter table `{0}` add  `source_id` varchar(40)	NOT NULL;'''.format(table))
            session.execute('''CREATE INDEX {0}_tmstamp_index ON `{0}` (tmstamp);'''.format(table))
            session.execute('''CREATE UNIQUE INDEX {0}_source_id_flag_index ON `{0}` (source_id,flag);'''.format(table))
        if unique_key is not None:
            unique_key.append('flag')
            self.create_unique_index(unique_key)
        session.commit()
        session.close()

    def create_unique_index(self, keys):
        session = self.dest_session()
        uni_key = ','.join(keys)
        sql = f'CREATE UNIQUE INDEX unique_{self.dest_table}_index ON `{self.dest_table}` ({uni_key})'
        session.execute(sql)
        session.commit()
        session.close()

    def get_delete_histroy_by_uplog(self, table, delete_log):
        sql = f"select ID,TableName,substring(KeyValue,8,10) as KeyValue from UpLog " \
              f"where TableName='{table}' and OperType='D' and ID>{delete_log};"
        return pd.read_sql(sql, self.source)

    def delete_by_uplog(self, source_table=None, delete_log=0):
        session = self.dest_session()
        if source_table is None:
            source_table = self.source_table
        delete_list = self.get_delete_histroy_by_uplog(source_table, delete_log)
        i, count = 0, 3000
        if not delete_list.empty:
            while i * count < len(delete_list):
                print(f'正在删除{self.dest_table}表{i * count}-{(i + 1) * count}的数据')
                slice_df = delete_list[i * count:(i + 1) * count]
                delete_sql = f"update {self.dest_table} set flag = null,is_verify=2 " \
                             f"where source_id in {list(slice_df['KeyValue'])} and flag=1".replace('[', '(').replace(
                    ']', ')')
                session.execute(delete_sql)
                session.commit()
                i += 1
            self.utils.update_update_log(delete_list['ID'].max(), '_del_' + self.dest_table)
        session.close()

    def _execute_dest_sql(self, sql):
        session = self.dest_session()
        session.execute(sql)
        session.commit()
        session.close()

    def delete_data_by_source_id(self, ids):
        start_time = time.time()
        i, count = 0, 5000
        while i * count < len(ids):
            del_ids = ids[i * count:(i + 1) * count]
            sql = '''update {0} set flag=null,is_verify=2 where source_id in {1} and flag=1'''.format(self.dest_table,
                                                                                                      del_ids)
            sql = sql.replace('[', '(').replace(']', ')')
            self._execute_dest_sql(sql)
            i = i + 1
        print('delete over.', time.time() - start_time)

    def save_update_log(self, task_name, max_tag):
        session = self.dest_session()
        sql = """insert into data_update_log (data_type,task_name,max_tag) values ('{0}','{1}','{2}') 
                        ON DUPLICATE KEY UPDATE data_type='{0}',task_name='{1}',max_tag='{2}',is_verify=2""".format(
            'basic_info',
            task_name,
            max_tag)
        session.execute(sql)
        session.commit()
        session.close()

    def insert_or_update(self, datas):
        session = self.dest_session()
        for i in range(datas.shape[0]):
            data = datas.iloc[i]
            values = ''
            update = ''
            title = ''
            for j in range(len(data)):
                index = data.index[j]
                value = str(data[j]).replace("'", "\\'").replace("%", "\\%").replace("(", "\(").replace(")",
                                                                                                        "\)").replace(
                    ":", "\:")
                title += """`{0}`,""".format(index)
                values += """'{0}',""".format(value)
                update += """`{0}`='{1}',""".format(index, value)

            sql = '''insert into {0} ({1}) values({2}) ON DUPLICATE KEY UPDATE {3}'''.format(self.dest_table,
                                                                                             title[0:-1],
                                                                                             values[0:-1],
                                                                                             update[0:-1]
                                                                                             )
            sql = sql.replace("'nan'", 'Null').replace("'None'", 'Null')
            try:
                session.execute(sql)
            except Exception as e:
                print(sql)
                print(e.orig.msg)
        session.commit()
        session.close()

    def save_to_mysql(self, df):
        tmpfile = self.dest_table + '_mysql.csv'
        sep = ','
        newline = '\n'
        start = time.time()
        self.insert_or_update(df[:0])
        # df[:0].to_sql(name=self.dest_table, con=self.destination, if_exists='append', index=False)
        columns = df.columns.values.tolist()
        print('creating ' + tmpfile, 'ok')
        with open(tmpfile, mode='w', newline=newline) as fhandle:
            df.to_csv(fhandle, na_rep='\\N', index=False, sep=sep)
        print('loading ' + tmpfile, 'ok')
        sql_load = "LOAD DATA LOCAL INFILE '{}' replace INTO TABLE {} FIELDS TERMINATED BY '{}' LINES TERMINATED BY '{}' IGNORE 1 LINES ({});" \
            .format(tmpfile, self.dest_table, sep, newline, ','.join(columns))
        self.destination.execute(sql_load)
        os.remove(tmpfile)
        end = time.time()
        print('存储数据库用时(s)：', end - start)
        return True

    def get_max_tm_source(self, table=None):
        if table is None:
            table = self.source_table
        sql = 'select max(update_time) as tm from ' + table
        trades_sets = pd.read_sql(sql, self.source)
        tm = 0
        if not trades_sets.empty:
            tm = trades_sets['tm'][0]
        return tm

    def get_max_tm_destation(self, table=None):
        if table == None:
            table = self.dest_table
        sql = 'select max(tmstamp)as tm from ' + table
        trades_sets = pd.read_sql(sql, self.destination)
        tm = 0
        if not trades_sets.empty:
            tm = trades_sets['tm'][0]
        return tm

    def get_max_tm_log(self, table=None):
        if table is None:
            table = self.dest_table
        sql = """select max_tag from update_log where task_name='{0}'""".format(table)
        trades_sets = pd.read_sql(sql, self.destination)
        tm = 0
        if not trades_sets.empty:
            tm = trades_sets['max_tag'][0]
        return tm

    def update_update_log(self, tm, table=None):
        if table is None:
            table = self.dest_table
        session = self.dest_session()
        sql = """insert into  update_log (task_name,max_tag) values ('{0}',{1}) 
                ON DUPLICATE KEY UPDATE task_name='{0}',max_tag={1}""".format(table, tm)
        sql = sql.replace('\n', '')
        session.execute(sql)
        session.commit()
        session.close()
        print('更新', table, '的max_tag为', tm)

    def get_fd_secode(self, keep='all'):
        sql = f'select security_code,fsymbol,end_date as _end_date from fd_basic_info where flag=1'
        result = pd.read_sql(sql, self.destination)
        result['_end_date'] = result['_end_date'].replace(datetime.date(1900, 1, 1), datetime.date(2099, 1, 1))
        result = result.sort_values(['fsymbol', '_end_date'])
        index = result.drop_duplicates(subset=['fsymbol'], keep='last').index
        result.loc[index, '_end_date'] = datetime.date(2099, 1, 1)

        if keep == 'last':
            result = result.drop_duplicates(subset=['fsymbol'], keep='last')
            result = result.drop(columns=['_end_date'])
        return result

    def get_manager_id(self, keep='all'):
        sql = f'select distinct ps_code as manager_code,ps_name as manager_name,security_code from fd_manager_info where flag=1'
        result = pd.read_sql(sql, self.destination)
        return result
