# coding=utf-8
import argparse
import pandas as pd
import numpy as np
from base_sync import BaseSync


class SyncFdManagerFactor(BaseSync):
    def __init__(self):
        super(SyncFdManagerFactor, self).__init__(source_table='hb_fof_fund_base_info',
                                                  dest_table='rl_fof_fund_base_info')

    def create_dest_tables(self):
        self.update_update_log(0)
        create_sql = """CREATE TABLE `{0}` (
                              `id`	INT UNSIGNED NOT NULL PRIMARY KEY AUTO_INCREMENT,
                              `security_code` varchar(40) DEFAULT NULL,
                              `fsymbol` varchar(40) DEFAULT NULL,
                              `rl_fund_code` varchar(40) default null comment '基金代码',
                              `rl_primary_class_code` varchar(40) default null comment 'rl一级分类code',
                              `rl_primary_class_name` varchar(40) default null comment 'rl一级分类name',
                              `rl_second_class_code` varchar(40) default null comment 'rl二级分类code',
                              `rl_second_class_name` varchar(40) default null comment 'rl二级分类name',
                              `entry_time` datetime DEFAULT NULL COMMENT '创建日期',
                              `is_valid` tinyint(1) DEFAULT '1' COMMENT '是否有效'
                            ) ENGINE=InnoDB DEFAULT CHARSET=utf8 COMMENT='持股集中度';
                            """.format(self.dest_table)
        create_sql = create_sql.replace('\n', '')
        create_sql = " ".join(create_sql.split())
        self.create_table(create_sql, model='basic_info')

    def get_sql(self):
        sql = """select
                    {0} as source_id,
                    left(HB_FUND_CODE,6) as fsymbol,
                    hb_fund_code as rl_fund_code,
                    hb_primary_class_code as rl_primary_class_code,
                    hb_primary_class_name as rl_primary_class_name,
                    hb_second_class_code as rl_second_class_code,
                    hb_second_class_name as rl_second_class_name,
                    create_time as entry_time,
                    update_time as tmstamp,
                    DELETE_FLAG as is_valid
                    from {1} a
                    where 1=1 and update_time > '{2}' and update_time<='{3}' and {0}>{4}
                    order by {0} limit 10000"""
        return sql

    def get_datas(self, tm, max_tm, id=0):
        print(f'正在查询 {self.source_table}表,update_time>={tm},id>{id}')
        sql = self.get_sql()
        sql = sql.format('ID', self.source_table, tm, max_tm, id).replace('\n', '')
        trades_sets = pd.read_sql(sql, self.source)
        trades_sets['is_valid'] = -(trades_sets['is_valid'] - 1)
        trades_sets.tmstamp = trades_sets.tmstamp.astype(int) / 1000000000
        return trades_sets

    def update_table_data(self, tm, max_tm):
        id = 0
        fd_secode = self.get_fd_secode(keep='last')
        while True:
            result_list = self.get_datas(tm, max_tm, id)
            result_list = pd.merge(result_list, fd_secode, on='fsymbol', how='left')
            result_list['flag'] = np.where(result_list.security_code.isna(),
                                           None,
                                           1)
            result_list.rl_primary_class_code = result_list.rl_primary_class_code.str[2:]
            result_list.rl_second_class_code = result_list.rl_second_class_code.str[2:]
            if not result_list.empty:
                try:
                    ids = list(result_list['source_id'])
                    self.delete_data_by_source_id(ids)
                    result_list.to_sql(name=self.dest_table, con=self.destination, if_exists='append', index=False)
                    id = ids[-1]
                except Exception as e:
                    print(e.orig.msg)
                    self.insert_or_update(result_list)
            else:
                self.update_update_log(max_tm.timestamp())
                break

    def do_update(self):
        max_tm = self.get_max_tm_source()
        log_tm = self.get_max_tm_log()
        log_tm = pd.to_datetime(log_tm * 1000000000)
        if max_tm > log_tm:
            self.update_table_data(log_tm, max_tm)


if __name__ == '__main__':
    parser = argparse.ArgumentParser()
    parser.add_argument('--rebuild', type=bool, default=True)
    parser.add_argument('--update', type=bool, default=False)
    args = parser.parse_args()
    if args.rebuild:
        processor = SyncFdManagerFactor()
        processor.create_dest_tables()
        processor.do_update()
    elif args.update:
        processor = SyncFdManagerFactor()
        processor.do_update()
