# coding=utf-8
import argparse
import pandas as pd
import numpy as np
from base_sync import BaseSync


class SyncFdManagerFactor(BaseSync):
    def __init__(self):
        super(SyncFdManagerFactor, self).__init__(source_table='hb_fof_multi_brinson_industry_attr_total',
                                                  dest_table='rl_fof_multi_brinson_industry_attr_total')

    def create_dest_tables(self):
        self.update_update_log(0)
        create_sql = """CREATE TABLE `{0}` (
                                      `id`	INT UNSIGNED NOT NULL PRIMARY KEY AUTO_INCREMENT,
                                      `security_code` varchar(40) DEFAULT NULL,
                                      `fsymbol` varchar(40) DEFAULT NULL,
                                      `benchmark` varchar(20) default null comment '基准',
                                      `data_type` varchar(8) default null comment '持仓类型(1:mainstockholding,2:allstockholding)',
                                      `fd_type` tinyint(1) default null comment '0 普通 1指数增强 2行业主题',
                                      `hb_cycle` varchar(8) default null comment '华宝周期',
                                      `level` tinyint(1) default null comment '行业级别',
                                      `start_date` varchar(8) default null comment '开始日期',
                                      `end_date` varchar(8) default null comment '结束日期',
                                      `ar` decimal(20,6) default null,
                                      `sr` decimal(20,6) default null,
                                      `ir` decimal(20,6) default null,
                                      `tr` decimal(20,6) default null,
                                      `entry_time` datetime DEFAULT NULL COMMENT '创建日期',
                                      `is_valid` tinyint(1) DEFAULT '1' COMMENT '是否有效',
                                      `disable` tinyint(1) DEFAULT NULL COMMENT '是否为正式数据 0为正式  1为tmp数据'
                                    ) ENGINE=InnoDB DEFAULT CHARSET=utf8 COMMENT='持股集中度';
                                    """.format(self.dest_table)
        create_sql = create_sql.replace('\n', '')
        create_sql = " ".join(create_sql.split())
        self.create_table(create_sql, model='basic_info')

    def get_sql(self):
        sql = """select
                    {0} as source_id,
                    left(HB_FUND_CODE,6) as fsymbol,
                    benchmark,
                    data_type,
                    type as fd_type,
                    hb_cycle,
                    level,
                    start_date,
                    end_date,
                    ar,
                    sr,
                    ir,
                    tr,
                    disable,
                    create_time as entry_time,
                    update_time as tmstamp,
                    DELETE_FLAG as is_valid
                    from {1} a
                    where 1=1 and update_time > '{2}' and update_time<='{3}' and {0}>{4}
                    order by {0} limit 10000"""
        return sql

    def get_datas(self, tm, max_tm, id=0):
        print(f'正在查询 {self.source_table}表,update_time>={tm},id>{id}')
        sql = self.get_sql()
        sql = sql.format('OBJECT_ID', self.source_table, tm, max_tm, id).replace('\n', '')
        trades_sets = pd.read_sql(sql, self.source)
        trades_sets['is_valid'] = -(trades_sets['is_valid'] - 1)
        trades_sets.tmstamp = trades_sets.tmstamp.astype(int) / 1000000000
        return trades_sets

    def update_table_data(self, tm, max_tm):
        id = 0
        fd_secode = self.get_fd_secode(keep='last')
        while True:
            result_list = self.get_datas(tm, max_tm, id)
            result_list = pd.merge(result_list, fd_secode, on='fsymbol', how='left')
            result_list['flag'] = np.where(result_list.security_code.isna(),
                                           None,
                                           1)
            if not result_list.empty:
                try:
                    ids = list(result_list['source_id'])
                    self.delete_data_by_source_id(ids)
                    result_list.to_sql(name=self.dest_table, con=self.destination, if_exists='append', index=False)
                    id = ids[-1]
                except Exception as e:
                    print(e.orig.msg)
                    self.insert_or_update(result_list)
            else:
                self.update_update_log(max_tm.timestamp())
                break

    def do_update(self):
        max_tm = self.get_max_tm_source()
        log_tm = self.get_max_tm_log()
        log_tm = pd.to_datetime(log_tm * 1000000000)
        if max_tm > log_tm:
            self.update_table_data(log_tm, max_tm)


if __name__ == '__main__':
    parser = argparse.ArgumentParser()
    parser.add_argument('--rebuild', type=bool, default=True)
    parser.add_argument('--update', type=bool, default=False)
    args = parser.parse_args()
    if args.rebuild:
        processor = SyncFdManagerFactor()
        processor.create_dest_tables()
        processor.do_update()
    elif args.update:
        processor = SyncFdManagerFactor()
        processor.do_update()
