#!/usr/bin/env python
# coding=utf-8
import numpy as np
import pandas as pd
import time
import warnings

warnings.filterwarnings("ignore")


class CalcTools(object):
    @classmethod
    def is_zero(cls, data_frame):
        return np.where(data_frame > -0.000001,
                        np.where(data_frame < 0.000001, True, False)
                        , False)

    def change_single(params):
        fundamentals_sets_year = params['fundamentals_sets_year']
        sub_columns = params['sub_columns']
        columns = fundamentals_sets_year.columns.values.tolist()
        miss_columns = list(set(columns) - set(sub_columns))

        def year_update(df):
            # df.loc[df.index, sub_columns] = df[sub_columns] - df[sub_columns].shift(-1).fillna(0)
            new_sub_df = df[sub_columns] - df[sub_columns].shift(1).fillna(0)
            new_df = df[miss_columns].join(new_sub_df)
            return new_df

        stock_list = list(set(fundamentals_sets_year['company_id']))
        new_fundamentals_sets = pd.DataFrame()
        i = 0
        for stock in stock_list:
            i += 1
            if i % 100 == 0:
                print(params['report_year'], ':', i, '/', len(stock_list))
            new_fundamentals_sets = new_fundamentals_sets.append(year_update(
                fundamentals_sets_year[
                    fundamentals_sets_year['company_id'] == stock]))
        return new_fundamentals_sets

    def change_single_by_symbol(params):
        fundamentals_sets = params['fundamentals_sets_symbol']
        fundamentals_sets.reset_index(drop=True, inplace=True)
        sub_columns = params['sub_columns']
        columns = fundamentals_sets.columns.values.tolist()
        miss_columns = list(set(columns) - set(sub_columns))

        def year_update(df):
            # df.loc[df.index, sub_columns] = df[sub_columns] - df[sub_columns].shift(-1).fillna(0)
            new_sub_df = df[sub_columns] - df[sub_columns].shift(1).fillna(0)
            new_df = df[miss_columns].join(new_sub_df)
            return new_df

        new_fundamentals_sets = pd.DataFrame()
        year_list = list(set(fundamentals_sets['report_year']))
        year_list.sort(reverse=True)
        stock_list = list(set(fundamentals_sets['company_id']))
        i = 0
        start = time.time()
        for stock in stock_list:
            i += 1
            if i % 100 == 0:
                end = time.time()
                print('cpu', params['cpu'], ':', i, '/', len(stock_list), ';time(s):', end - start)
                start = end
            for year in year_list:
                fundamentals_sets_stock = fundamentals_sets[fundamentals_sets['company_id'] == stock]
                new_fundamentals_sets = new_fundamentals_sets.append(year_update(
                    fundamentals_sets_stock[
                        fundamentals_sets_stock['report_year'] == year]))
        return new_fundamentals_sets

    # df必须是以PUBLISHDATE倒序排布
    def get_line(df, pub_date):
        df = df[df['PUBLISHDATE'] <= pub_date]
        if df.empty:
            return pd.Series()
        return df.iloc[0]

    # 按照发布日期计算mrq
    @classmethod
    def df_calc_mrq_by_publish_date(self, df1, df2, static_columns, sub_columns):
        df = pd.DataFrame()
        pub_date_list = list(set(df1['PUBLISHDATE']).union(df2['PUBLISHDATE']))
        for pub_date in pub_date_list:
            line1 = self.get_line(df1, pub_date)
            line2 = self.get_line(df2, pub_date)
            if not line1.empty:
                if not line2.empty:
                    line = pd.concat([line1[static_columns], line1[sub_columns] - line2[sub_columns]])
                    line['PUBLISHDATE'] = pub_date
                    if line1['REPORTTYPE'] == '1' and line2['REPORTTYPE'] == '1':
                        line['REPORTTYPE'] = '1'
                    else:
                        line['REPORTTYPE'] = '3'
                # 若无上季度数据，则将需要计算单季的字段置空，记录保留
                else:
                    line1[sub_columns] = None
                    line = line1
                df = df.append(line, ignore_index=True)
        return df

    # 按照发布日期填充数据,若存在report_type的1和3是同一天发布，即数据相同的情况下只保留1的数据
    @classmethod
    def fill_by_order_publish_date(self, df1, df2, columns):
        if df2.empty or df1.empty:
            return df1
        else:
            df = pd.DataFrame()
            pub_date_list = list(set(df1['PUBLISHDATE']).union(df2['PUBLISHDATE']))
            for pub_date in pub_date_list:
                line1 = self.get_line(df1, pub_date)
                line2 = self.get_line(df2, pub_date)
                if not line1.empty:
                    if not line2.empty:
                        line1[columns] = line2[columns]
                        line1['PUBLISHDATE'] = pub_date
                    df = df.append(line1)
        return df

    def fill_na_by_before(params):
        fundamentals_sets = params['fundamentals_sets_symbol']
        half_fill_date_arr = params['half_fill_date_arr']
        year_fill_date_arr = params['year_fill_date_arr']
        half_year_columns = params['half_year_columns']
        year_columns = params['year_columns']
        fundamentals_sets.sort_values(by=['COMPCODE', 'ENDDATE', 'PUBLISHDATE', 'REPORTTYPE'],
                                      ascending=[False, False, False, True],
                                      inplace=True)
        fundamentals_groupd = fundamentals_sets.groupby(['COMPCODE'])
        # 记录需要补充的未填充报告期数据
        supplement_half, supplement_year = [], []
        df = pd.DataFrame(half_fill_date_arr)
        if not df.empty:
            supplement_half = list(set(df[1]))
        df = pd.DataFrame(year_fill_date_arr)
        if not df.empty:
            supplement_year = list(set(df[1]))

        df = pd.DataFrame()
        i = 0
        start = time.time()
        for k, g in fundamentals_groupd:
            i += 1
            if i % 100 == 0:
                end = time.time()
                print('cpu', params['cpu'], ':', i, '/', len(fundamentals_groupd), ';time(s):', end - start)
                start = end
            df_fill_half_year = pd.DataFrame()
            for fill_date in half_fill_date_arr:
                try:
                    fill_df = CalcTools.fill_by_order_publish_date(g[g['ENDDATE'] == fill_date[0]],
                                                                   g[g['ENDDATE'] == fill_date[1]],
                                                                   half_year_columns)
                    df_fill_half_year = df_fill_half_year.append(fill_df)
                except Exception as e:
                    print(e)
            # fill_df只包含被填充过的报告期数据，无需填充的部分在此批量添加
            df_fill_half_year = df_fill_half_year.append(g[g['ENDDATE'].isin(supplement_half)])
            df_fill_year = pd.DataFrame()
            for fill_date in year_fill_date_arr:
                try:
                    fill_df = CalcTools.fill_by_order_publish_date(
                        df_fill_half_year[df_fill_half_year['ENDDATE'] == fill_date[0]],
                        df_fill_half_year[df_fill_half_year['ENDDATE'] == fill_date[1]],
                        year_columns)
                    df_fill_year = df_fill_year.append(fill_df)
                except Exception as e:
                    print(e)
            # fill_df只包含被填充过的报告期数据，无需填充的部分在此批量添加
            df_fill_year = df_fill_year.append(g[g['ENDDATE'].isin(supplement_year)])
            if not df_fill_year.empty:
                df = df.append(df_fill_year)
            else:
                df = df.append(df_fill_half_year)
        return df

    def mrq_by_symbol(params):
        fundamentals_sets = params['fundamentals_sets_symbol']
        fundamentals_sets.reset_index(drop=True, inplace=True)
        sub_columns = params['sub_columns']
        half_year_sub_columns = params['half_year_columns']
        year_sub_columns = params['year_columns']
        mrq_sub_columns = list(set(params['sub_columns']) - set(half_year_sub_columns) - set(year_sub_columns))
        columns = fundamentals_sets.columns.values.tolist()
        static_columns_out_sub = list(set(columns) - set(mrq_sub_columns))
        miss_columns_out_half_sub = list(set(columns) - set(half_year_sub_columns))
        # fundamentals_sets.sort_values(by=['COMPCODE', 'ENDDATE', 'PUBLISHDATE'], ascending=False, inplace=True)
        fundamentals_sets.sort_values(by=['COMPCODE', 'ENDDATE', 'PUBLISHDATE', 'REPORTTYPE'],
                                      ascending=[False, False, False, True],
                                      inplace=True)

        def year_update(df):
            ds = df.sort_values(by='REPORTDATETYPE')
            report_date_type = list(set(df['REPORTDATETYPE']))
            new_sub_df = pd.DataFrame(columns=sub_columns)
            if '1' in report_date_type:
                new_sub_df = new_sub_df.append(ds[ds['REPORTDATETYPE'] == '1'])
                report_date_type.remove('1')
            for i in report_date_type:
                try:
                    # 计算所有可能出现的单季情况
                    sub_s = CalcTools.df_calc_mrq_by_publish_date(ds[ds['REPORTDATETYPE'] == i],
                                                                  ds[ds['REPORTDATETYPE'] == str(int(i) - 1)],
                                                                  static_columns_out_sub,
                                                                  mrq_sub_columns)
                    # 计算半年报字段单季数据
                    if i == '4' and not sub_s.empty:
                        if '2' in report_date_type:
                            sub_s = CalcTools.df_calc_mrq_by_publish_date(sub_s, ds[ds['REPORTDATETYPE'] == '2'],
                                                                          miss_columns_out_half_sub,
                                                                          half_year_sub_columns)
                        else:
                            sub_s[half_year_sub_columns] = None
                    new_sub_df = new_sub_df.append(sub_s, ignore_index=True)
                except Exception as e:
                    print(e)
                    new_sub_df = new_sub_df.append({}, ignore_index=True)
            new_sub_df.reset_index(inplace=True, drop=True)
            return new_sub_df

        new_fundamentals_sets = pd.DataFrame()
        year_list = list(set(fundamentals_sets['REPORTYEAR']))
        year_list.sort(reverse=True)
        stock_list = list(set(fundamentals_sets['COMPCODE']))
        i = 0
        start = time.time()
        for stock in stock_list:
            i += 1
            if i % 100 == 0:
                end = time.time()
                print('cpu', params['cpu'], ':', i, '/', len(stock_list), ';time(s):', end - start)
                start = end
            for year in year_list:
                fundamentals_sets_stock = fundamentals_sets[fundamentals_sets['COMPCODE'] == stock]
                new_fundamentals_sets = new_fundamentals_sets.append(year_update(
                    fundamentals_sets_stock[
                        fundamentals_sets_stock['REPORTYEAR'] == year]))
        return new_fundamentals_sets

    @classmethod
    def df_calc_ttm_by_publish_date(self, df1, df2, df3, static_columns, sub_columns):
        df = pd.DataFrame()
        pub_date_list = list(set(df1['PUBLISHDATE']).union(df2['PUBLISHDATE']).union(df3['PUBLISHDATE']))
        for pub_date in pub_date_list:
            line1 = self.get_line(df1, pub_date)
            line2 = self.get_line(df2, pub_date)
            line3 = self.get_line(df3, pub_date)
            if not line1.empty:
                if not line2.empty and not line3.empty:
                    line = pd.concat(
                        [line1[static_columns], line1[sub_columns] + line2[sub_columns] - line3[sub_columns]])
                    line['PUBLISHDATE'] = pub_date
                    if line1['REPORTTYPE'] == '1' and line2['REPORTTYPE'] == '1' and line3['REPORTTYPE'] == '1':
                        line['REPORTTYPE'] = '1'
                    else:
                        line['REPORTTYPE'] = '3'
                # 若无上季度数据，则将需要计算单季的字段置空，记录保留
                else:
                    line1[sub_columns] = None
                    line = line1
                df = df.append(line, ignore_index=True)
        return df

    def ttm_by_symbol(params):
        fundamentals_sets = params['fundamentals_sets_symbol']
        fundamentals_sets.reset_index(drop=True, inplace=True)
        # fundamentals_sets = fundamentals_sets.sort_values(by='ENDDATE', ascending=False)
        fundamentals_sets.sort_values(by=['COMPCODE', 'ENDDATE', 'PUBLISHDATE', 'REPORTTYPE'],
                                      ascending=[False, False, False, True],
                                      inplace=True)
        sub_columns = params['sub_columns']
        columns = fundamentals_sets.columns.values.tolist()
        miss_columns = list(set(columns) - set(sub_columns))

        def year_update(df, pre_year_df):
            report_date_type = list(df['REPORTDATETYPE'])
            pre_report_date_type = list(pre_year_df['REPORTDATETYPE'])
            new_sub_df = pd.DataFrame()
            if '4' in report_date_type:
                new_sub_df = new_sub_df.append(df[df['REPORTDATETYPE'] == '4'])
                report_date_type.remove('4')
            if '4' in pre_report_date_type:
                for i in report_date_type:
                    try:
                        sub_s = CalcTools.df_calc_ttm_by_publish_date(df[df['REPORTDATETYPE'] == i],
                                                                      pre_year_df[pre_year_df['REPORTDATETYPE'] == '4'],
                                                                      pre_year_df[pre_year_df['REPORTDATETYPE'] == i],
                                                                      miss_columns,
                                                                      sub_columns)
                        new_sub_df = new_sub_df.append(sub_s, ignore_index=True)
                    except Exception as e:
                        print(e)
            else:
                sub_s = df[df['REPORTDATETYPE'] != '4']
                sub_s[sub_columns] = None
                new_sub_df = new_sub_df.append(sub_s, ignore_index=True)
            new_sub_df.reset_index(drop=True, inplace=True)
            return new_sub_df

        new_fundamentals_sets = pd.DataFrame()
        year_list = list(set(fundamentals_sets['REPORTYEAR']))
        year_list.sort(reverse=True)
        stock_list = list(set(fundamentals_sets['COMPCODE']))

        i = 0
        start = time.time()
        for stock in stock_list:
            i += 1
            if i % 100 == 0:
                end = time.time()
                print('cpu', params['cpu'], ':', i, '/', len(stock_list), ';time(s):', end - start)
                start = end
            fundamentals_sets_stock = fundamentals_sets[fundamentals_sets['COMPCODE'] == stock]
            for j in range(len(year_list) - 1):
                new_fundamentals_sets = new_fundamentals_sets.append(
                    year_update(fundamentals_sets_stock[fundamentals_sets_stock['REPORTYEAR'] == year_list[j]],
                                fundamentals_sets_stock[fundamentals_sets_stock['REPORTYEAR'] == year_list[j + 1]]))
        return new_fundamentals_sets

    # df必须是以PUBLISHDATE倒序排布
    def get_fin_line(df, pub_date):
        df = df[df['publish_date'] <= pub_date]
        if df.empty:
            return pd.Series()
        return df.iloc[0]

    @classmethod
    def df_calc_fin_ttm_by_publish_date(self, df1, df2, df3, static_columns, sub_columns):
        df = pd.DataFrame()
        pub_date_list = list(set(df1['publish_date']).union(df2['publish_date']).union(df3['publish_date']))
        for pub_date in pub_date_list:
            line1 = self.get_fin_line(df1, pub_date)
            line2 = self.get_fin_line(df2, pub_date)
            line3 = self.get_fin_line(df3, pub_date)
            if not line1.empty:
                if not line2.empty:
                    if not line3.empty:
                        line = pd.concat(
                            [line1[static_columns], line1[sub_columns] + line2[sub_columns] - line3[sub_columns]])
                        line['publish_date'] = pub_date
                        line['report_type'] = line1['report_type']
                        # 若去年年报不为空，则对ttm为null的值做填充
                        line[sub_columns] = line[sub_columns].fillna(line2[sub_columns])
                        df = df.append(line, ignore_index=True)
                    # 20200511修改：若无去年年报或去年同期数据，则不计算当期ttm数据
                    """
                    else:
                        line = pd.concat([line1[static_columns], line2[sub_columns]])
                        line['publish_date'] = pub_date
                        line['report_type'] = line1['report_type']
                        # print('无去年同期数据；', line1['company_id'], ',', line1['report_date'])
                # 若无去年年报数据，则将需要计算单季的字段置空，记录保留
                else:
                    line1[sub_columns] = None
                    line1['publish_date'] = pub_date
                    line = line1
                df = df.append(line, ignore_index=True)
                """
        return df

    def fin_ttm_by_symbol(params):
        fundamentals_sets = params['fundamentals_sets_symbol']
        fundamentals_sets.sort_values(by=['company_id', 'report_date', 'publish_date', 'report_type'],
                                      ascending=[False, False, False, True],
                                      inplace=True)
        fundamentals_sets.reset_index(drop=True, inplace=True)
        sub_columns = params['sub_columns']
        columns = fundamentals_sets.columns.values.tolist()
        miss_columns = list(set(columns) - set(sub_columns))

        def year_update(df, pre_year_df):
            report_date_type = list(set(df['report_date_type']))
            pre_report_date_type = list(set(pre_year_df['report_date_type']))
            new_sub_df = pd.DataFrame()
            if '4' in report_date_type:
                new_sub_df = new_sub_df.append(df[df['report_date_type'] == '4'])
                report_date_type.remove('4')
            if '4' in pre_report_date_type:
                for i in report_date_type:
                    try:
                        # 分别计算合并报表和母公司报表的ttm
                        for report_types in ['1', '2']:
                            sub_s = CalcTools.df_calc_fin_ttm_by_publish_date(
                                df[(df['report_date_type'] == i) & (df['report_type'] == report_types)],
                                pre_year_df[
                                    (pre_year_df['report_date_type'] == '4') & (
                                            pre_year_df['report_type'] == report_types)],
                                pre_year_df[
                                    (pre_year_df['report_date_type'] == i) & (
                                            pre_year_df['report_type'] == report_types)],
                                miss_columns,
                                sub_columns)
                            new_sub_df = new_sub_df.append(sub_s, ignore_index=True)
                    except Exception as e:
                        print(e)
            # else:
            #     sub_s = df[df['report_date_type'] != '4']
            #     sub_s[sub_columns] = None
            #     new_sub_df = new_sub_df.append(sub_s, ignore_index=True)
            new_sub_df.reset_index(drop=True, inplace=True)
            return new_sub_df

        new_fundamentals_sets = pd.DataFrame()
        year_list = list(set(fundamentals_sets['report_year']))
        year_list.sort(reverse=True)
        stock_list = list(set(fundamentals_sets['company_id']))

        i = 0
        start = time.time()
        for stock in stock_list:
            i += 1
            if i % 100 == 0:
                end = time.time()
                print('cpu', params['cpu'], ':', i, '/', len(stock_list), ';time(s):', end - start)
                start = end
            fundamentals_sets_stock = fundamentals_sets[fundamentals_sets['company_id'] == stock]
            for j in range(len(year_list) - 1):
                new_fundamentals_sets = new_fundamentals_sets.append(
                    year_update(fundamentals_sets_stock[fundamentals_sets_stock['report_year'] == year_list[j]],
                                fundamentals_sets_stock[fundamentals_sets_stock['report_year'] == year_list[j + 1]]))
        return new_fundamentals_sets

    @classmethod
    def df_calc_fin_mrq_by_publish_date(self, df1, df2, static_columns, sub_columns):
        df = pd.DataFrame()
        pub_date_list = list(set(df1['publish_date']).union(df2['publish_date']))
        for pub_date in pub_date_list:
            line1 = self.get_fin_line(df1, pub_date)
            line2 = self.get_fin_line(df2, pub_date)
            if not line1.empty:
                if not line2.empty:
                    line = pd.concat(
                        [line1[static_columns], line1[sub_columns] - line2[sub_columns]])
                    line['publish_date'] = pub_date
                    line['report_type'] = line1['report_type']
                    df = df.append(line, ignore_index=True)
                # 若无上一季度数据，则将需要计算单季的字段置空，记录保留
                # 20200511修改：若无去年年报数据，则不计算当期单季数据
                """
                else:
                    line1[sub_columns] = None
                    line = line1
                """
        return df

    def fin_mrq_by_symbol(params):
        fundamentals_sets = params['fundamentals_sets_symbol']
        fundamentals_sets.sort_values(by=['company_id', 'report_date', 'publish_date', 'report_type'],
                                      ascending=[False, False, False, True],
                                      inplace=True)
        fundamentals_sets.reset_index(drop=True, inplace=True)
        sub_columns = params['sub_columns']
        columns = fundamentals_sets.columns.values.tolist()
        miss_columns = list(set(columns) - set(sub_columns))

        def year_update(df):
            report_date_type = list(set(df['report_date_type']))
            new_sub_df = pd.DataFrame()
            if '1' in report_date_type:
                new_sub_df = new_sub_df.append(df[df['report_date_type'] == '1'])
                report_date_type.remove('1')
            for i in report_date_type:
                try:
                    # 分别计算合并报表和母公司报表的ttm
                    for report_types in ['1', '2']:
                        sub_s = CalcTools.df_calc_fin_mrq_by_publish_date(
                            df[(df['report_date_type'] == i) & (df['report_type'] == report_types)],
                            df[(df['report_date_type'] == str(int(i) - 1)) & (df['report_type'] == report_types)],
                            miss_columns,
                            sub_columns)
                        new_sub_df = new_sub_df.append(sub_s, ignore_index=True)
                except Exception as e:
                    print(e)
            new_sub_df.reset_index(drop=True, inplace=True)
            return new_sub_df

        new_fundamentals_sets = pd.DataFrame()
        year_list = list(set(fundamentals_sets['report_year']))
        year_list.sort(reverse=True)
        stock_list = list(set(fundamentals_sets['company_id']))

        i = 0
        start = time.time()
        for stock in stock_list:
            i += 1
            if i % 100 == 0:
                end = time.time()
                print('cpu', params['cpu'], ':', i, '/', len(stock_list), ';time(s):', end - start)
                start = end
            fundamentals_sets_stock = fundamentals_sets[fundamentals_sets['company_id'] == stock]
            for j in range(len(year_list) - 1):
                new_fundamentals_sets = new_fundamentals_sets.append(
                    year_update(fundamentals_sets_stock[fundamentals_sets_stock['report_year'] == year_list[j]]))
        return new_fundamentals_sets

    def mrq_by_symbol(params):
        fundamentals_sets = params['fundamentals_sets_symbol']
        fundamentals_sets.reset_index(drop=True, inplace=True)
        sub_columns = params['sub_columns']
        half_year_sub_columns = params['half_year_columns']
        year_sub_columns = params['year_columns']
        mrq_sub_columns = list(set(params['sub_columns']) - set(half_year_sub_columns) - set(year_sub_columns))
        columns = fundamentals_sets.columns.values.tolist()
        static_columns_out_sub = list(set(columns) - set(mrq_sub_columns))
        miss_columns_out_half_sub = list(set(columns) - set(half_year_sub_columns))
        # fundamentals_sets.sort_values(by=['COMPCODE', 'ENDDATE', 'PUBLISHDATE'], ascending=False, inplace=True)
        fundamentals_sets.sort_values(by=['COMPCODE', 'ENDDATE', 'PUBLISHDATE', 'REPORTTYPE'],
                                      ascending=[False, False, False, True],
                                      inplace=True)

        def year_update(df):
            ds = df.sort_values(by='REPORTDATETYPE')
            report_date_type = list(set(df['REPORTDATETYPE']))
            new_sub_df = pd.DataFrame(columns=sub_columns)
            if '1' in report_date_type:
                new_sub_df = new_sub_df.append(ds[ds['REPORTDATETYPE'] == '1'])
                report_date_type.remove('1')
            for i in report_date_type:
                try:
                    # 计算所有可能出现的单季情况
                    sub_s = CalcTools.df_calc_mrq_by_publish_date(ds[ds['REPORTDATETYPE'] == i],
                                                                  ds[ds['REPORTDATETYPE'] == str(int(i) - 1)],
                                                                  static_columns_out_sub,
                                                                  mrq_sub_columns)
                    # 计算半年报字段单季数据
                    if i == '4' and not sub_s.empty:
                        if '2' in report_date_type:
                            sub_s = CalcTools.df_calc_mrq_by_publish_date(sub_s, ds[ds['REPORTDATETYPE'] == '2'],
                                                                          miss_columns_out_half_sub,
                                                                          half_year_sub_columns)
                        else:
                            sub_s[half_year_sub_columns] = None
                    new_sub_df = new_sub_df.append(sub_s, ignore_index=True)
                except Exception as e:
                    print(e)
                    new_sub_df = new_sub_df.append({}, ignore_index=True)
            new_sub_df.reset_index(inplace=True, drop=True)
            return new_sub_df

        new_fundamentals_sets = pd.DataFrame()
        year_list = list(set(fundamentals_sets['REPORTYEAR']))
        year_list.sort(reverse=True)
        stock_list = list(set(fundamentals_sets['COMPCODE']))
        i = 0
        start = time.time()
        for stock in stock_list:
            i += 1
            if i % 100 == 0:
                end = time.time()
                print('cpu', params['cpu'], ':', i, '/', len(stock_list), ';time(s):', end - start)
                start = end
            for year in year_list:
                fundamentals_sets_stock = fundamentals_sets[fundamentals_sets['COMPCODE'] == stock]
                new_fundamentals_sets = new_fundamentals_sets.append(year_update(
                    fundamentals_sets_stock[
                        fundamentals_sets_stock['REPORTYEAR'] == year]))
        return new_fundamentals_sets
