import argparse
import re
from io import BytesIO
from obs import *

import pandas as pd

'''
  This sample demonstrates how to do bucket-related operations
  (such as do bucket ACL/CORS/Lifecycle/Logging/Website/Location/Tagging/OPTIONS)
  on OBS using the OBS SDK for Python.
'''

config = {
    'AK': 'QATS9AJH5NZVTREENE25',
    'SK': 'jCIfH6U5Atv7xwBB4yY7xcynf7aimnhqlGcDoiDD',
    'server': 'obs.cn-east-3.myhuaweicloud.com',
    'bucket_name': 'csvstore'
}


class ObsUtil():
    def __init__(self):
        self._obs_client = ObsClient(access_key_id=config['AK'], secret_access_key=config['SK'],
                                     server=config['server'])
        # self._bucket_client = self._obs_client.bucketClient(config['bucket_name'])

    def push_df_to_bucket(self, df, file_path):
        df_string = BytesIO()
        df_string.write(df.to_csv(index=False, sep=',', encoding='UTF-8').encode('utf-8'))
        df_string.seek(0, 0)
        return self.push_stream_to_bucket(df_string, file_path)

    def push_stream_to_bucket(self, stream, file_path):
        resp = self._obs_client.putContent(config['bucket_name'], file_path, content=stream)
        if resp.status < 300:
            print('requestId:' + resp.requestId)
            print(f'save {file_path} success!')
            return True
        else:
            print('errorCode:' + resp.errorCode)
            print('errorMessage:' + resp.errorMessage)
            print(f'save {file_path} faild!')
            return False

    def exist_obj(self, file_path):
        resp = self._obs_client.getObjectMetadata(config['bucket_name'], file_path)
        if resp.status < 300:
            return True
        else:
            return False

    def push_df_to_bucket_from_file(self, df, file_path, bucketname=None):
        file_name = f'/tmp/{str(time.time())}.csv'
        df.to_csv(file_name, index=False, sep=',', encoding='UTF-8')
        return self.push_file_to_bucket(file_name, file_path, bucketname)

    def push_file_to_bucket(self, file_path, obs_file_path, bucketname=None):
        if bucketname is None:
            bucketname = config.bucket_name

        resp = self._obs_client.putFile(bucketname, obs_file_path, file_path)
        if resp.status < 300:
            print('requestId:' + resp.requestId)
            print(f'save {file_path} success!')
            return True
        else:
            print('errorCode:' + resp.errorCode)
            print('errorMessage:' + resp.errorMessage)
            print(f'save {file_path} faild!')
            return False


if __name__ == '__main__':
    parser = argparse.ArgumentParser()
    parser.add_argument('--path', type=str, default='/Users/zzh/Downloads/20210404.csv')
    args = parser.parse_args()
    util = ObsUtil()
    # df = pd.DataFrame({
    #     "a": [1, 2, 3],
    #     "b": [4, 5, 6],
    #     "c": [7, 8, 9]
    # })
    # 测试路径
    # util.push_df_to_bucket(df, 'test/20200101.csv')
    # 真实路径
    # util.push_df_to_bucket(df, 'TFZQ/tf_exception/20200101.csv')
    obs_path = re.split('[/\\\\]', args.path)
    obs_path = [i for i in obs_path if i != '']
    obs_path = ['TFZQ'] + obs_path[1:]
    obs_path = '/'.join(obs_path)
    util.push_file_to_bucket(args.path, obs_path, 'csvstore')
