from io import BytesIO
from obs import *

import config
import pandas as pd
import time

'''
  This sample demonstrates how to do bucket-related operations
  (such as do bucket ACL/CORS/Lifecycle/Logging/Website/Location/Tagging/OPTIONS)
  on OBS using the OBS SDK for Python.
'''


# log = LoggerFactory.create_engine()


class ObsUtil():
    def __init__(self):
        self._obs_client = ObsClient(access_key_id=config.AK, secret_access_key=config.SK,
                                     server=config.server)
        # self._bucket_client = self._obs_client.bucketClient(config.bucket_name)

    def push_df_to_bucket(self, df, file_path):
        df_string = BytesIO()
        df_string.write(df.to_csv(index=False, sep=',', encoding='UTF-8').encode('utf-8'))
        df_string.seek(0, 0)
        return self.push_stream_to_bucket(df_string, file_path)

    def push_df_to_bucket_from_file(self, df, file_path, bucketname=None):
        file_name = f'/tmp/{str(time.time())}.csv'
        df.to_csv(file_name, index=False, sep=',', encoding='utf_8_sig')
        return self.push_file_to_bucket(file_name, file_path, bucketname)

    def push_stream_to_bucket(self, stream, file_path):
        resp = self._obs_client.putContent(config.bucket_name, file_path, content=stream)
        if resp.status < 300:
            print('requestId:' + resp.requestId)
            print(f'save {file_path} success!')
            return True
        else:
            print('errorCode:' + resp.errorCode)
            print('errorMessage:' + resp.errorMessage)
            print(f'save {file_path} faild!')
            return False

    def push_file_to_bucket(self, file_path, obs_file_path, bucketname=None):
        if bucketname is None:
            bucketname = config.bucket_name

        resp = self._obs_client.putFile(bucketname, obs_file_path, file_path)
        if resp.status < 300:
            print('requestId:' + resp.requestId)
            print(f'save {file_path} success!')
            return True
        else:
            print('errorCode:' + resp.errorCode)
            print('errorMessage:' + resp.errorMessage)
            print(f'save {file_path} faild!')
            return False

    # 二进制下载数据
    def get_bitdata_from_bucket(self, file_path):
        resp = self._obs_client.getObject(config.bucket_name, file_path, loadStreamInMemory=True)
        if resp.status < 300:
            print('url:' + resp.body.url)
            # df = pd.read_csv(io.BytesIO(resp.body.buffer), encoding='gb2312', sep=",")
            # df = pd.read_excel(io.BytesIO(resp.body.buffer), encoding='gb2312', sep=",")
        else:
            print('errorCode:' + resp.errorCode)
            print('errorMessage:' + resp.errorMessage)
        return resp.body.buffer

    # 文件下载数据
    def get_file_from_bucket(self, file_path, save_path):
        resp = self._obs_client.getObject(config.bucket_name, file_path, downloadPath=save_path)
        if resp.status < 300:
            print('requestId:' + resp.requestId)
            print('url:' + resp.body.url)
            return True
        else:
            print('errorCode:' + resp.errorCode)
            print('errorMessage:' + resp.errorMessage)
            return False

    def get_stream_from_bucket(self, file_path):
        result = b''
        resp = self._obs_client.getObject(config.bucket_name, file_path, loadStreamInMemory=False)
        if resp.status < 300:
            print('requestId:' + resp.requestId)
            # 读取对象内容
            while True:
                chunk = resp.body.response.read(10485760)
                if not chunk:
                    break
                else:
                    result += chunk
            resp.body.response.close()
        else:
            print('errorCode:' + resp.errorCode)
            print('errorMessage:' + resp.errorMessage)
        return result

    def get_dataframe_from_bucket(self, file_path, type='csv', encoding='utf-8'):
        df = pd.DataFrame()
        stream = self.get_stream_from_bucket(file_path)
        if len(stream) > 0:
            if type == 'csv':
                df = pd.read_csv(BytesIO(stream), encoding=encoding, sep=",")
            elif type == 'excel':
                df = pd.read_excel(BytesIO(stream), encoding=encoding)
        return df

    def get_obj_metadata(self, file_path):
        resp = self._obs_client.getObjectMetadata(config.bucket_name, file_path)
        if resp.status < 300:
            print('requestId:', resp.requestId)
            print('contentType:', resp.body.contentType)
            print('contentLength:', resp.body.contentLength)
            print('property:', dict(resp.header).get('property'))
        else:
            print('requestId:', resp.requestId)

    def exist_obj(self, file_path):
        resp = self._obs_client.getObjectMetadata(config.bucket_name, file_path)
        if resp.status < 300:
            return True
        else:
            return False


if __name__ == '__main__':
    import time

    util = ObsUtil()
    start = time.time()
    # util.get_stream_from_bucket('factor_tf_securities_init.xlsx')
    # result = util.get_stream_from_bucket('0617.csv')
    # df = pd.read_csv(io.BytesIO(result), encoding='gb2312', sep=",")
    # util.get_dataframe_from_bucket('stand_factor_data/factor_inhouse_000985_1w.csv')
    # util.get_dataframe_from_bucket('TFZQ/tf_fund_factor/fund_product/20201130.csv',encoding='GBK')
    # util.get_dataframe_from_bucket('QSJG/QSJG202011.csv', encoding='GBK')
    # print(time.time() - start)
    # start = time.time()
    # util.get_file_from_bucket('factor_tf_securities_init.xlsx', './factor_tf_securities_init.xlsx')
    # print(time.time() - start)
    # util.exist_obj('stand_factor_data1/')
    util.get_dataframe_from_bucket('tfzq/industry/20210420/20210420_industry_data1.csv')
